unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls;

type
  TMainForm = class(TForm)
    PollTime: TTimer;
    StatusLabel: TLabel;
    LogMemo: TMemo;
    Panel1: TPanel;
    NewLogButton: TButton;
    ConfigButton: TButton;
    AcqButton: TButton;
    procedure FormCreate(Sender: TObject);
    procedure PollTimeTimer(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure NewLogButtonClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure ConfigButtonClick(Sender: TObject);
    procedure AcqButtonClick(Sender: TObject);
    procedure StopButtonClick(Sender: TObject);
  protected
    procedure LogEndLog;
    { Private declarations }
  public
    procedure DoTare(Channel: Integer);
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  AIOWDM, ConfigUnit, ChannelConfigFrameUnit;

const
  clGreyBase = $010101;
  MaxFlash = 51;
  clFlashBase = clGreyBase * 5;

var
  CardNum: LongWord = $FFFFFFFF;
  NextSample: Integer;
  SampleData: array of Double;
  FN: String;
  FlashTime: Byte;
  Scans: Integer;
  NextTime, dTime: LongWord;

procedure CtrMode(CardNum: LongWord; Ctr, Mode: byte);
var
  ModeCmd: Byte;

begin
  ModeCmd := (Ctr shl 6) or $30 or (mode shl 1);
  RelOutPortB(CardNum, $B, ModeCmd);
end;

procedure CtrLoad(CardNum: LongWord; Ctr, Value: word);
begin
  RelOutPortB(CardNum, 8+Ctr, Value and $00FF);
  RelOutPortB(CardNum, 8+Ctr, (Value shr 8) and $00FF);
end;

procedure StopAndReset(CardNum: LongWord);
begin
  //Stop the counters.
  CtrMode(CardNum, 1, 2);
  CtrMode(CardNum, 2, 2);

  RelOutPortB(CardNum, 4, $48); //40h resets channel fifo, 08h resets data fifo.
end;

procedure StopAndPrime(CardNum: LongWord);
begin
  //Stop the counters.
  CtrMode(CardNum, 1, 2);
  CtrMode(CardNum, 2, 2);

  RelOutPortB(CardNum, 4, $09); //08h resets data fifo, 01h enables conversion on counter.
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
  I: Integer;
  DeviceID: LongWord;

begin
  for I := 0 to GetNumCards - 1 do begin
    if QueryCardInfo(I, @DeviceID, nil, nil, nil) = 0 then Continue;

    if DeviceID <> $ACA9 then Continue;

    CardNum := I;
    Break;
  end;

  if CardNum = $FFFFFFFF then begin
    Caption := 'PAcq - No PCI-AI12-16A Found';

    //`@ Complain.

    Application.ShowMainForm := False;
    Application.Terminate;
    Exit;
  end;

  StatusLabel.Color := $000000;
  StatusLabel.Font.Color := $000000;

  LogMemo.Lines.Add('EZACCES slice in Windows for Alcoa');
  LogMemo.Lines.Add('Acquires from a PCI-AI12-16A and logs to .CSV');
end;

procedure TMainForm.PollTimeTimer(Sender: TObject);

  procedure SaveString(S: String);
  var
    Fil: TFileStream;

  begin
    Fil := TFileStream.Create(FN, fmOpenReadWrite or fmShareDenyNone);
    Fil.Seek(0, soFromEnd);
    Fil.Write(S[1], Length(S));
    Fil.Free;
  end;

var
  I: Integer;
  Buf: String;
  bGotData: Boolean;
  ST: TSystemTime;

begin
  if GetTickCount >= NextTime then begin
    CtrLoad(CardNum, 1, 2);
    CtrLoad(CardNum, 2, 5);
    Inc(NextTime, dTime);
  end;

  bGotData := False;
  while (RelInPortB(CardNum, 4) and $02) <> 0 do begin
    I := RelInPort(CardNum, 0) and $0FFF;
    with ChannelData[NextSample] do SampleData[NextSample] := (I - Tare) * Scale;
    Inc(NextSample);
    if NextSample > 12 then begin
      StopAndPrime(CardNum);
      NextSample := 0;
      bGotData := True;

      GetLocalTime(ST);
      Buf := Format('%.2d/%.2d,%.2d:%.2d:%.2d', [ ST.wMonth, ST.wDay, ST.wHour, ST.wMinute, ST.wSecond ]);
      for I := 0 to 12 do begin
        if I = 8 then
          SampleData[I] := SampleData[I] * 0.18 + 32
        ;
        Buf := Buf + Format(',%.7f', [ SampleData[I] ]);
      end;
      Buf := Buf + #13#10;
      try
        SaveString(Buf);
      except
        LogMemo.Lines.Add('Error writing to log');
        NewLogButton.Click;
        SaveString(Buf);
      end;

      Inc(Scans);
      StatusLabel.Caption := 'Got Data x' + IntToStr(Scans);
    end;
  end;

  if bGotData then
    FlashTime := MaxFlash
  else if FlashTime <> 0 then
    Dec(FlashTime)
  else
    Exit
  ;
  StatusLabel.Font.Color := clFlashBase * FlashTime;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  StopAndReset(CardNum);
end;

procedure TMainForm.NewLogButtonClick(Sender: TObject);
var
  ST: TSystemTime;
  Fil: TFileStream;
  Buf: String;
  I: Integer;

begin
  GetLocalTime(ST);
  if Length(FN) <> 0 then LogEndLog;
  FN := Format('%.4d%.2d%.2d %.2d%.2d%.2d.csv', [ ST.wYear, ST.wMonth, ST.wDay, ST.wHour, ST.wMinute, ST.wSecond ]);
  LogMemo.Lines.Add('New log started: ' + FN);
  Fil := TFileStream.Create(FN, fmCreate);
  Buf := '"Date","Time",';
  for I := 0 to 7 do
    Buf := Buf + '"LVDT' + IntToStr(I + 1) + '",'
  ;
  Buf := Buf + '"Temp(F)","AIM3","AIM5","AIM7","AIM8"'#13#10;
  Fil.Write(Buf[1], Length(Buf));
  Fil.Free;

  Scans := 0;
end;

procedure TMainForm.LogEndLog;
begin
  LogMemo.Lines.Add('Current log ended with ' + IntToStr(Scans) + ' readings');
end;

procedure TMainForm.FormResize(Sender: TObject);
begin
  //NewLogButton.Left := ClientWidth - 8 - NewLogButton.Width;
end;

procedure TMainForm.ConfigButtonClick(Sender: TObject);
begin
  ConfigForm.Show;
end;

procedure TMainForm.AcqButtonClick(Sender: TObject);
var
  I: Integer;
  
begin
  ConfigForm.Visible := False;

  NewLogButton.Click;

  StopAndReset(CardNum);
  for I := 0 to High(ChannelData) do
    RelOutPort(CardNum, 2, ChannelData[I].Code)
  ;
  {
  for I := 0 to 7 do
    RelOutPort(CardNum, 2, I * $1000)
  ;
  RelOutPort(CardNum, 2, $0014); //Temperature reference.
  RelOutPort(CardNum, 2, $2010);
  RelOutPort(CardNum, 2, $4010);
  RelOutPort(CardNum, 2, $6010);
  RelOutPort(CardNum, 2, $7010);
  }
  RelInPortB(CardNum, 2);

  dTime := ConfigForm.IntervalUpDown.Position * 100;
  NextTime := GetTickCount + dTime;
  RelOutPortB(CardNum, 4, $01); //01h enables conversion on counter.

  SetLength(SampleData, 13);

  NextSample := 0;
  PollTime.Interval := 100;//Round(StrToFloat(ConfigForm.IntervalEdit.Text) * 1000);
  PollTime.Enabled := True;
  LogMemo.Lines.Add('Acquiring...');

  ConfigButton.Enabled := False;
  NewLogButton.Enabled := True;
  AcqButton.Caption := '&Stop';
  AcqButton.OnClick := StopButtonClick;
end;

procedure TMainForm.StopButtonClick(Sender: TObject);
begin
  StopAndReset(CardNum);
  LogEndLog;
  FN := '';

  PollTime.Enabled := False;

  ConfigButton.Enabled := True;
  NewLogButton.Enabled := False;
  AcqButton.Caption := '&Acquire';
  AcqButton.OnClick := AcqButtonClick;
end;

procedure TMainForm.DoTare(Channel: Integer);
begin
  StopAndReset(CardNum);
  RelOutPort(CardNum, 2, ChannelData[Channel].Code);
  RelOutPort(CardNum, 2, ChannelData[Channel].Code);
  RelInPortB(CardNum, 2);
  RelOutPortB(CardNum, 0, $00);
  while (RelInPortB(CardNum, 4) and $02) = 0 do ;
  ChannelData[Channel].Tare := RelInPort(CardNum, 0) and $0FFF;
end;

end.
